// Time-stamp: <06-Sep-2002 15:55:20 jwerner>
/*====================================================================
//
// cetus2gpl.cc - Convert a Cetus Track file to GPL file
//
//     by: John E. Werner
//         02-Sep-2002
//
//====================================================================
//
// Purpose:
//
// System Subsystem:
//         Engineer: John E. Werner
//
//--------------------------------------------------------------------
//
//  Copyright (C) 1998, 2000, 2001, 2002 John Werner, All Rights Reserved.
//
// Copyright protection claimed includes all forms and matters of
// copyrightable material and information now allowed by statutory or
// judicial law or hereinafter granted, including without limitation,
// material generated from the software program which are displayed on
// the screen such as icons, screen display looks, etc.
//
//--------------------------------------------------------------------
//
// Revision History:
//   02-Sep-2002  JEWerner	Created.
//
//------------------------------------------------------------------*/

#include <iostream.h>
#include <fstream.h>
#include <string>
#include <time.h>
#include "gpldata.h"
#include "tracklog.h"
#include "track.h"

typedef unsigned short UShort;
typedef unsigned long  ULong;

#define KmhPerKnot        1.851999
#define KmhPerMph         1.609344
#define MphPerKnot        1.150779
#define FeetPerMeter      3.280840


char * GPS_Type_Names[] =
{
	"No GPS detected",
	"Generic GPS",
	"Demo GPS",
   "Expansion card GPS",
	"Garmin GPS",
	"Rockwell GPS",
	"Magellan GPS",
	"DeLorme Tripmate GPS",
	"Motorola GPS"
};
/* Interval texts */
char *Interval_Texts[] =
{
	"All positions",
	"Movements",
	"Every second",
        "Every 2 seconds",
	"Every 10 seconds",
	"Every minute"
};

//--------------------------------------------------------------------
// main(...)
//
//   Description:
//	The main program file
//
//   Params:
//	void
//
//   Returns:
//	int	0	no error
//		!0	Error
//			-1 	Failed to open input file
//			-2	Failed to open output file
//			-3	Invalid arguments
//--------------------------------------------------------------------

int main(int argc, char **argv)
{
   if (argc != 3)
   {
      cout << "Usage: cetus2gpl <input file.PDB> <output file.GPL>" << endl;
      return(-3);
   }

   std::string outName, inName;

   inName = argv[1];

   cout << "Opening " << inName << endl;
   ifstream in(inName.c_str(), ios::binary);
   if (!in)
   {
      cout << "\"" << inName << "\" - Open Failed!!!" << endl;
      return -1;
   }


   outName = argv[2];				// get the file name
   cout << "Opening " << outName << endl;
   ofstream out(outName.c_str(), ios::binary | ios::trunc);
   if (!out)
   {
      cout << "\"" << outName << "\" - Open Failed!!!" << endl;
      in.close();
      return(-2);
   }

   cout << "Working...";
   int  num = 1;
   if (!in.eof())				// see if input file done
   {
      int err = 0;
      struct init_type init;
      struct rec_type rec;

      /* first we try to read the track init information */
      cout << "Finding Data block start..." << endl;

      err = track_init(in, &init);

      cout << "\tDescription:\t" << init.desc << endl
           << "\tTrack Strated on:\t" << init.day << "-" << init.month << "-"
           << init.year << " @ " << init.hour << ":" << init.minute << ":"
           << init.second << " UTC" << endl
           << "\tTime Zone Offset:\t" << init.timeZone << endl
           << "\tRecord Interval:\t" << Interval_Texts[init.interval] << endl
           << "\tGPS Type:\t\t" << GPS_Type_Names[init.gps] << endl;



      /* if we got the information */
      if (!err)
      {
         cout << "Processing..." << endl;

         /* uncomment this if you want UTC rather than local timezone */
         /* init.timeZone = 0; */

//		/* convert time and date to local timezone */
//		adjust_init_time (&init);

         /* do something about the init data here */

         /* export the track records, stop when reaching EOF */
         while (!in.eof())
         {
            track_record(in, rec);

//			   if (rec.fix >= 2)
//				   adjust_record_time (&init, &rec);

            struct  tm time_data;

            time_data.tm_hour = rec.hour + init.timeZone;
            time_data.tm_min= rec.minute;
            time_data.tm_sec = rec.second;
            time_data.tm_mday = init.day;
            time_data.tm_year = init.year + 2000 - 1900;
            time_data.tm_mon =  init.month - 1;

            GPL_Data_Point	data(rec.lat / 10000000.0,
                                rec.lon / 10000000.0,
                                rec.altitude * FeetPerMeter / 100.0,
                                rec.course / 100.0,
                                rec.speed * MphPerKnot * 10.0 / 100.0,
                                mktime(&time_data),
                                (ULong) rec.fix);

            out.write((char *) &data, sizeof(data));
            out.flush();

            num++;
            if (num % 100 == 0)
               cout << ".";

         } // while (!in.eof())
      } // if (!err)
   } // if (!in.eof())

   // close our files
   in.close();
   out.close();

   cout << endl
        << "Wrote " << num << " records." << endl;

   return 0;
} // End
