// Time-stamp: <05-Sep-2002 13:22:27 jwerner>
/*====================================================================
//
// findstop.cc - Find the stops in a GPL file
//
//     by: John E. Werner
//         03-Feb-1998
//
//====================================================================
//
// Purpose:
//
// System Subsystem:
//         Engineer: John E. Werner
//
//--------------------------------------------------------------------
//
//  Copyright (C) 1998, 2001, 2002 John Werner, All Rights Reserved.
//
// Copyright protection claimed includes all forms and matters of
// copyrightable material and information now allowed by statutory or
// judicial law or hereinafter granted, including without limitation,
// material generated from the software program which are displayed on
// the screen such as icons, screen display looks, etc.
//
//--------------------------------------------------------------------
//
// Revision History:
//   03-Feb-1998  JEWerner	Created.
//
//------------------------------------------------------------------*/

#pragma pack(1)

#include <iostream.h>
#include <fstream.h>
#include <string>
#include <stdlib.h>
#include "gpldata.h"

typedef unsigned short UShort;
typedef unsigned long  ULong;


//--------------------------------------------------------------------
// main(...)
//
//   Description:
//	The main execution loop
//
//   Params:
//	int	argc	- the number of arguments
//	char	**argv	- the argnuments
//
//   Returns:
//	int	0	No Error
//		!0	Error
//			-1 	Failed to open input file
//			-2	Failed to open output file
//			-3 	Invalid arguments
//--------------------------------------------------------------------

int main(int argc, char **argv)
{
   if ((argc < 3) || (argc > 4))		// check number of args
   {
      cout << "Usage: findstop <input.gpl> <output.gpl> [<stop speed>]"
           << endl
           << endl
           << "where:\t<input.gpl>   is the GPL file to read" << endl
           << "\t<output.gpl>  is the file to write" << endl
           << "\t<stop speed>  is the max speed to consider stopped (0.0)"
           << endl
           << endl
           << "Example:\tfindstop input.gpl output.gpl 3.0" << endl
           << "\t This reads from input.gpl, writes output.gpl, and considers"
           << endl << "\t anything less then 3.0 mph to be stopped." << endl;
      return -3;
   }
   double stopSpeed(0.0);
   bool   speedGiven = false;    

   if (argc == 4)				// speed given?
   {
      stopSpeed = strtod(argv[3], NULL);	// get it
      speedGiven = true;
   }    

   std::string      outName(argv[2]), inName(argv[1]);

   cout << "Opening " << inName << endl;
   ifstream in(inName.c_str(), ios::binary);
   if (!in)
   {
      cout << "\"" << inName << "\" - Open Failed!!!" << endl;
      return -1;
   }

   cout << "Opening " << outName << endl;
   ofstream out(outName.c_str(), ios::binary | ios::trunc);
   if (!out)
   {
      cout << "\"" << outName << "\" - Open Failed!!!" << endl;
      in.close();
      return(-2);
   }

   cout << "Working...";
   bool done = false;
   int  num = 1;
   int  written = 0;
   bool justWrote = false;    

   GPL_Data_Point oldData;
   GPL_Data_Point data;
   while (!done)
   {
      bool writeIt = false;

      if (in.eof())				// see if input file done
      {
         done = true;
         out.write((char *) &oldData, sizeof(oldData)); // write last point
         break;
      }

      oldData = data;				// save last read

      // get data point
      in.read((char *) &data, sizeof(data));
      if (in.fail())
      {
         done = true;
         out.write((char *) &oldData, sizeof(oldData)); // write last point
         break;
      }

      if (num == 1)				// first point?
         writeIt = true;			// ... yes, so write it
      else if (!speedGiven && data.SamePlace(oldData)) // see if same place
         writeIt = true;			// ... yes, so write it
      else if (data.GetSpeed() <= stopSpeed)	// stop speed?
         writeIt = true;
      else
         justWrote = false;			// write again.
	

      if (!justWrote && writeIt)		// need to be written?
      {
         justWrote = true;	    
         out.write((char *) &data, sizeof(data)); // ... yes, so write it
         written++;
      }

      num++;
      if (num % 100 == 0)
         cout << ".";

   } // while(!done)

   // close stuff and leave
   in.close();
   out.close();

   cout << "done." << endl;
   cout << "Read " << num << " data points. Wrote " << written << " stops." 
        << endl;

   return 0;
} // main(...)
