// Time-stamp: <05-Sep-2002 13:21:19 jwerner>
/*====================================================================
//
// gpl2vrml.cc - Take a gpl file and make a vrml file out of it!
//
//     by: John Werner
//	   22-Oct-1997
//
//====================================================================
//
// Purpose:
//
// System Subsystem:
//	   Engineer: John E. Werner
//
//--------------------------------------------------------------------
//
//  Copyright (C) 1997, 1998, 2000, 2001, 2002 John E. Werner, All Rights Reserved.
//
// Copyright protection claimed includes all forms and matters of
// copyrightable material and information now allowed by statutory or
// judicial law or hereinafter granted, including without limitation,
// material generated from the software program which are displayed on
// the screen such as icons, screen display looks, etc.
//
//--------------------------------------------------------------------
//
// Revision History:
//   22-Oct-1997  JEWerner	Created.
//
//------------------------------------------------------------------*/

#include "gpldata.h"
#include "GFC.h"
#include <string>
#include <stdlib.h>
#include <stdio.h>
//#include <io.h>
#include <fcntl.h>

#define READ_SIZE	100
#define FT_PER_METER	3.281

//--------------------------------------------------------------------
// main
//
//   Description:
//	the main function
//
//   Params:
//	int	argc
//	char	*argv[]
//
//   Returns:
//	int	0	no errors
//		!0	errors
//--------------------------------------------------------------------

int main(int argc, char **argv)
{
   double   scalex = 1.0;
   double   scaley = 1.0;
   double   scalez = 1.0;
   double   scaleAlt = 1.0;
   double   scaleLat = 1.0;
   double   scaleLon = 1.0;
   int j;

   if (argc < 2)
   {
      cout << "Usage: graphgpl [-sx <scale_x>] [-sy <scale_y>] [-sz <scale_z>] [-salt <scale_alt>] [-slat <scale_lat>] [-slon <scale_lon>]"
         " <file1> <file2> .... <filen>" << endl;
      return(-2);
   }

   // Look for scale
   for (j = 1; j < argc; j++)
   {
      if ((std::string) argv[j] == "-sx")
         scalex = strtod(argv[++j], 0);
      else if ((std::string) argv[j] == "-sy")
         scaley = strtod(argv[++j], 0);
      else if ((std::string) argv[j] == "-sz")
         scalez = strtod(argv[++j], 0);
      else if ((std::string) argv[j] == "-salt")
         scaleAlt = strtod(argv[++j], 0);
      else if ((std::string) argv[j] == "-slat")
         scaleLat = strtod(argv[++j], 0);
      else if ((std::string) argv[j] == "-slon")
         scaleLon = strtod(argv[++j], 0);
      else
         break;
   }

   // write the header of the program
   cout << "#VRML V2.0 utf8" << endl
        << "#" << endl
        << "WorldInfo {" << endl
        << "  info [" << endl
        << "	 \"Created By: gpl2vrml"
            " by: John Werner <flybrick@frontiernet.net>\"" << endl
        << "  ]" << endl
        << "  info [" << endl
        << "    \"(X,Y,Z) Scaling: (" << scalex << ", " << scaley << ", "
        << scalez << ")\"" << endl
        << "  ]" << endl
        << "  info [" << endl
        << "    \"(Lon,Lat,Alt) Scaling: (" << scaleLon << ", " << scaleLat
        << ", " << scaleAlt << ")\"" << endl
        << "  ]" << endl
        << "}" << endl;

   std::string	fileName;

   // j is set above from processing or arguments
   for (; j < argc; j++)
   {
      fileName = argv[j];

      FILE *handle = fopen (fileName.c_str(), "rw");
      if (handle == NULL)
      {
         cerr << "\"" << fileName << "\" - Open Failed!!!" << endl;
         return(-1);
      }

      cout << "# Data from: " << fileName << endl
           << "Transform {" << endl
           << "    translation 0 0 0" << endl
           << "    scale 1 1 1" << endl
           << "    rotation 1 0 0 0" << endl
           << "    children [" << endl
           << "	 Shape {" << endl
           << "	   appearance Appearance {" << endl
           << "	     material Material {" << endl
           << "	       diffuseColor	0.8 0.8 0 # cyan" << endl
           << "	       emissiveColor	0.9 0.9 0.0" << endl
           << "	       ambientIntensity 0.1" << endl
           << "	       shininess	0.1" << endl
           << "	       transparency	0.2" << endl
           << "	     }" << endl
           << "	   }" << endl
           << "	   geometry IndexedLineSet {" << endl
           << "		 coord Coordinate { point [" << endl;

      bool	done = false;
      int	num = 1;
      int	count = 0;
      CEarth		earth;
      CPolarCoordinate	tempCoord;
      CEarthCoordinate	currentCoord;
      CEarthCoordinate	startCoord;
      GPL_Data_Point	startPoint;
      GPL_Data_Point	lastPoint;
      double	minx = 0.0, maxx = 0.0;
      double	miny = 0.0, maxy = 0.0;
      double	minz = 0.0, maxz = 0.0;

      while (!done)
      {
         GPL_Data_Point	data[READ_SIZE];
         int size = fread (data, sizeof(GPL_Data_Point), READ_SIZE, handle);
         int number = size;

         for (GPL_Data_Point *dataPntr = data;
              dataPntr < data + number;
              dataPntr++, num++)
         {
            double x, y, z;

            // check to see if we have new place and output
            if ((num == 1) || (!dataPntr->SamePlace(lastPoint)))
            {
               tempCoord.Set(dataPntr->GetLattitude() * scaleLat,
                             dataPntr->GetLongitude() * scaleLon,
                             dataPntr->GetAltitude() * scaleAlt / FT_PER_METER);

               earth.Convert(tempCoord, currentCoord);

               if (num == 1)
               {
                  startPoint = *dataPntr;
                  startCoord =  currentCoord;
               } // if (num == 1)

               currentCoord = currentCoord - startCoord;

//		    cout << dataPntr->GetLongitude() << "  "
//			 << dataPntr->GetLattitude() << "   "
//			 << dataPntr->GetAltitude() << "," << endl;

               x = currentCoord.GetXCoordinateInMeters() * scalex;
               y = currentCoord.GetYCoordinateInMeters() * scaley;
               z = currentCoord.GetZCoordinateInMeters() * scalez;

               if (x > maxx) maxx = x;
               if (x < minx) minx = x;
               if (y > maxy) maxy = y;
               if (y < miny) miny = y;
               if (z > maxz) maxz = z;
               if (z < minz) minz = z;

               cout << x << "  " << y << "	 " << z << ","
                    << "\t# " << num << endl;

               // count 1 more
               count++;
            } // if (!dat...)


            lastPoint = *dataPntr;

            if (feof(handle))
               done = true;
         }
      } // while (!done)
      fclose(handle);				// done with file

      cout << "		 ]}" << endl
           << "		 coordIndex [" << endl;
      count --;				// adjust start
      for (; count >= 0; count --)
      {
         cout << count << ", " ;
         if ((count % 15) == 0)
            cout << endl;
      }
      cout << "-1" << endl;

      cout << "		 ]" << endl
           << "		 colorPerVertex FALSE" << endl
           << endl
           << "}}]}" << endl;

      // make	 marker
      cout << "# Start Marker" << endl
           << "Transform {" << endl
           << "    translation 0 0 0"
           << "    scale 1 1 1" << endl
           << "    rotation 1 0 0 0" << endl
           << "    children [" << endl
           << "	 Shape {" << endl
           << "	   geometry Box { size 10 10 10} " << endl
           << "	   appearance Appearance {" << endl
           << "	     material Material {" << endl
           << "	       diffuseColor	0 0.8 0.8 # cyan" << endl
           << "	       emissiveColor	0.0 0.0 0.0" << endl
           << "	       ambientIntensity 0.1" << endl
           << "	       shininess	0.1" << endl
           << "	       transparency	0.2" << endl
           << "	     }" << endl
           << "	   }" << endl
           << "	 }" << endl
           << "   ]" << endl
           << " }" << endl;

      // make end marker
      cout << "# END Marker" << endl
           << "Transform {" << endl
           << "    translation "
           << currentCoord.GetXCoordinateInMeters() << "  "
           << currentCoord.GetYCoordinateInMeters() << "  "
           << currentCoord.GetZCoordinateInMeters() << endl
           << "    scale 1 1 1" << endl
           << "    rotation 1 0 0 0" << endl
           << "    children [" << endl
           << "	 Shape {" << endl
           << "	   geometry Box { size 10 10 10} " << endl
           << "	   appearance Appearance {" << endl
           << "	     material Material {" << endl
           << "	       diffuseColor	0.8 0.8 0 # cyan" << endl
           << "	       emissiveColor	0.0 0.0 0.0" << endl
           << "	       ambientIntensity 0.1" << endl
           << "	       shininess	0.1" << endl
           << "	       transparency	0.2" << endl
           << "	     }" << endl
           << "	   }" << endl
           << "	 }" << endl
           << "   ]" << endl
           << " }" << endl;

      // make bounding box
      cout << "# Bounding box"<< endl
           << "Transform {" << endl
           << "    translation 0 0 0" << endl
           << "    scale 1 1 1" << endl
           << "    rotation 1 0 0 0" << endl
           << "    children [" << endl
           << "	 Shape {" << endl
           << "	   appearance Appearance {" << endl
           << "	     material Material {" << endl
           << "	       diffuseColor	0 0.8 0.8 # cyan" << endl
           << "	       emissiveColor	0.8 0.0 0.8" << endl
           << "	       ambientIntensity 0.1" << endl
           << "	       shininess	0.1" << endl
           << "	       transparency	0.2" << endl
           << "	     }" << endl
           << "	   }" << endl
           << "	   geometry IndexedLineSet {" << endl
           << "		 coord Coordinate { point [" << endl
           << minx << " " << miny << " " << minz << "," << endl
           << minx << " " << miny << " " << maxz << "," << endl
           << minx << " " << maxy << " " << minz << "," << endl
           << minx << " " << maxy << " " << maxz << "," << endl
           << maxx << " " << miny << " " << minz << "," << endl
           << maxx << " " << miny << " " << maxz << "," << endl
           << maxx << " " << maxy << " " << minz << "," << endl
           << maxx << " " << maxy << " " << maxz << endl
           << "		 ]}" << endl
           << "		 coordIndex [" << endl
           << "0, 2, 6, 4, 0, -1," << endl
           << "1, 3, 7, 5, 1, -1," << endl
           << "0, 1, -1," << endl
           << "2, 3, -1," << endl
           << "4, 5, -1," << endl
           << "6, 7, -1" << endl
           << "		 ]" << endl
           << "		 colorPerVertex FALSE" << endl
           << endl
           << "}}]}" << endl;

      // Axis
      cout << "# Axis" << endl
           << "Transform {" << endl
           << "    translation 0 0 0" << endl
           << "    scale 1 1 1" << endl
           << "    rotation 1 0 0 0" << endl
           << "    children [" << endl
           << "	 Shape {" << endl
           << "	   appearance Appearance {" << endl
           << "	     material Material {" << endl
           << "	       diffuseColor	0 0.8 0.8 # cyan" << endl
           << "	       emissiveColor	0.0 0.8 0.8" << endl
           << "	       ambientIntensity 0.1" << endl
           << "	       shininess	0.1" << endl
           << "	       transparency	0.2" << endl
           << "	     }" << endl
           << "	   }" << endl
           << "	   geometry IndexedLineSet {" << endl
           << "		 coord Coordinate { point [" << endl
           << "0 0 " << minz << "," << endl
           << "0 0 " << maxz << "," << endl
           << "0 " << miny << " 0," << endl
           << "0 " << maxy << " 0," << endl
           << minx << " 0 0," << endl
           << maxx << " 0 0"  << endl
           << "		 ]}" << endl
           << "		 coordIndex [" << endl
           << "0, 1, -1," << endl
           << "2, 3, -1," << endl
           << "4, 5, -1" << endl
           << "		 ]" << endl
           << "		 colorPerVertex FALSE" << endl
           << endl
           << "}}]}" << endl;


   } // for (j = 1; ...)

   return(0);
} // int main(...)
