// Time-stamp: <05-Sep-2002 17:30:14 jwerner>
/*====================================================================
//
// gpldata.cc - Class Methods for DeLorme GPL Data Class
//
//     by: John Werner x25054
//         22-Oct-1997
//
//====================================================================
//
// Purpose:
//
// System Subsystem:
//         Engineer: John E. Werner
//
//--------------------------------------------------------------------
//
//  Copyright (C) 1997, 1998, 2000, 2001, 2002 John E. Werner, All Rights Reserved.
//
// Copyright protection claimed includes all forms and matters of
// copyrightable material and information now allowed by statutory or
// judicial law or hereinafter granted, including without limitation,
// material generated from the software program which are displayed on
// the screen such as icons, screen display looks, etc.
//
//--------------------------------------------------------------------
//
// Revision History:
//   02-Sep-2002  JEWerner Fixed processing of text string so that
//                         we can build under gcc.
//   22-Oct-1997  JEWerner      Created.
//
//------------------------------------------------------------------*/

#include "gpldata.h"
#include <time.h>
#include <iostream.h>
#include <string>
#include <stdio.h>
#include <time.h>


//--------------------------------------------------------------------
// GPL_Data_Point::GPL_Data_Point()
//
//   Description:
//      default constructor
//
//   Params:
//      void
//
//   Returns:
//
//--------------------------------------------------------------------
GPL_Data_Point::GPL_Data_Point()
  : status(0), dummy(0), lattitude(0), longitude(0), altitude(0),
    heading(0), speed(0), time(0), dummy3(0)
{
} // GPL_Data_Point()

//--------------------------------------------------------------------
// GPL_Data_Point::GPL_Data_Point(double, double)
//
//   Description:
//      default constructor
//
//   Params:
//      double  lat     - the lattitude
//      double  lon     - the longitude
//
//   Returns:
//
//--------------------------------------------------------------------
GPL_Data_Point::GPL_Data_Point(double lat, double lon)
  : status(0), dummy(0), lattitude(lat), longitude(lon), altitude(0),
    heading(0), speed(0), time(0), dummy3(0)
{
} // GPL_Data_Point()

//--------------------------------------------------------------------
// GPL_Data_Point::GPL_Data_Point(double, double, double)
//
//   Description:
//      default constructor
//
//   Params:
//      double  lat     - the lattitude
//      double  lon     - the longitude
//      double  alt     - the altitude
//
//   Returns:
//
//--------------------------------------------------------------------
GPL_Data_Point::GPL_Data_Point(double lat, double lon, double alt)
  : status(0), dummy(0), lattitude(lat), longitude(lon), altitude(alt),
    heading(0), speed(0), time(0), dummy3(0)
{
} // GPL_Data_Point()

//--------------------------------------------------------------------
// GPL_Data_Point::GPL_Data_Point(double, double, double, double)
//
//   Description:
//      default constructor
//
//   Params:
//      double  lat     - the lattitude
//      double  lon     - the longitude
//      double  alt     - the altitude
//      double  head    - the heading
//
//   Returns:
//
//--------------------------------------------------------------------
GPL_Data_Point::GPL_Data_Point(double lat, double lon, double alt, double head)
  : status(0), dummy(0), lattitude(lat), longitude(lon), altitude(alt),
    heading(head), speed(0), time(0), dummy3(0)
{
} // GPL_Data_Point()

//--------------------------------------------------------------------
// GPL_Data_Point::GPL_Data_Point(double, double, double, double, double spd)
//
//   Description:
//      default constructor
//
//   Params:
//      double  lat     - the lattitude
//      double  lon     - the longitude
//      double  alt     - the altitude
//      double  head    - the heading
//      double  spd     - the speed
//
//   Returns:
//
//--------------------------------------------------------------------
GPL_Data_Point::GPL_Data_Point(double lat, double lon, double alt, double head,
                               double spd)
  : status(0), dummy(0), lattitude(lat), longitude(lon), altitude(alt),
    heading(head), speed(spd), time(0), dummy3(0)
{
} // GPL_Data_Point()

//--------------------------------------------------------------------
// GPL_Data_Point::GPL_Data_Point(double, double, double, double, double spd)
//
//   Description:
//      default constructor
//
//   Params:
//      double  lat     - the lattitude
//      double  lon     - the longitude
//      double  alt     - the altitude
//      double  head    - the heading
//      double  spd     - the speed
//      ULong   t       - the time
//
//   Returns:
//
//--------------------------------------------------------------------
GPL_Data_Point::GPL_Data_Point(double lat, double lon, double alt, double head,
                               double spd, ULong t)
  : status(0), dummy(0), lattitude(lat), longitude(lon), altitude(alt),
    heading(head), speed(spd), time(t), dummy3(0)
{
} // GPL_Data_Point()

//--------------------------------------------------------------------
// GPL_Data_Point::GPL_Data_Point(double, double, double, double, double spd)
//
//   Description:
//      default constructor
//
//   Params:
//              double  lat     - the lattitude
//              double  lon     - the longitude
//              double  alt     - the altitude
//              double  head    - the heading
//              double  spd     - the speed
//              ULong   t       - the time
//      ULong status
//
//   Returns:
//
//--------------------------------------------------------------------
GPL_Data_Point::GPL_Data_Point(double lat, double lon, double alt, double head,
                               double spd, ULong t, ULong stat)
  : status(stat), dummy(0), lattitude(lat), longitude(lon), altitude(alt),
    heading(head), speed(spd), time(t), dummy3(0)
{
} // GPL_Data_Point()

//--------------------------------------------------------------------
// GPL_Data_Point::GPL_Data_Point(GPL_Data_Point &right)
//
//   Description:
//      Copy Constructor
//
//   Params:
//      const GPL_Data_Point &right
//
//   Returns:
//      const GPL_Data_Point &
//--------------------------------------------------------------------
GPL_Data_Point::GPL_Data_Point(const GPL_Data_Point &right)
{
    status      = right.status;
    dummy       = right.dummy;
    lattitude   = right.lattitude;
    longitude   = right.longitude;
    altitude    = right.altitude;
    heading     = right.heading;
    speed       = right.speed;
    time        = right.time;
    dummy3      = right.dummy3;
} // GPL_Data_Point(const GPL_Data_Point &)

//--------------------------------------------------------------------
// GPL_Data_Point::operator<<
//
//   Description:
//      stream output function for our class
//
//   Params:
//      ostream &               os
//      const GPL_Data_Point &  dt
//
//   Returns:
//      ostream &
//--------------------------------------------------------------------
ostream& operator<< (ostream& os, const GPL_Data_Point& dt)
{
    char buffer[80];

    strftime(buffer, 80, "%d-%b-%Y, %H:%M:%S",
             localtime((const time_t *) &dt.time));

    // set formatting (printf sure looks nice right now)
    os.setf( ios::fixed );
    os.setf( ios::showpoint );
    os.precision(8);
    os << dt.status << ", " << dt.lattitude << ", " << dt.longitude << ", "
       << buffer << ", ";
    os.precision(2);
    os.width(6);
    os << dt.heading << ", ";
    os.precision(5);
    os.width(8);
    os << dt.speed;
    os << ", " << dt.altitude;
    return os;
} // operator<<(...)

//--------------------------------------------------------------------
// GPL_Data_Point::SamePlace
//
//   Description:
//      Compares the location of two data points
//
//   Params:
//      GPL_Data_Point& dt
//
//   Returns:
//      bool    true    - they match
//              false   - no match
//--------------------------------------------------------------------
bool GPL_Data_Point::SamePlace(const GPL_Data_Point& dt) const
{
    bool result = ((lattitude == dt.lattitude) &&
                   (longitude == dt.longitude)) ? true : false;

    return result;
} // GPL_Data_Point::SamePlace(...)

//--------------------------------------------------------------------
// GPL_Data_Point::operator==
//
//   Description:
//      equality operator
//
//   Params:
//      const GPL_Data_Point &right
//
//   Returns:
//      int     0       !=
//              !0      ==
//--------------------------------------------------------------------
int GPL_Data_Point::operator== (const GPL_Data_Point& right) const
{
    return ((lattitude == right.lattitude) &&
            (longitude == right.longitude) &&
            (status == right.status) &&
//          (dummy == right.dummy) &&
            (altitude == right.altitude) &&
            (heading == right.heading) &&
            (speed == right.speed) &&
            (time == right.time)
//          (dummy3 == right.dummy3)
            );

} // GPL_Data_Point::operator==

//--------------------------------------------------------------------
// GPL_Data_Point::operator!=
//
//   Description:
//      equality operator
//
//   Params:
//      const GPL_Data_Point &right
//
//   Returns:
//      int     0       ==
//              !0      !=
//--------------------------------------------------------------------
int GPL_Data_Point::operator!= (const GPL_Data_Point& right) const
{
    return ((lattitude != right.lattitude) &&
            (longitude != right.longitude) &&
            (status != right.status) &&
//          (dummy != right.dummy) &&
            (altitude != right.altitude) &&
            (heading != right.heading) &&
            (speed != right.speed) &&
            (time != right.time)
//          (dummy3 != right.dummy3)
            );

} // operator!=

//--------------------------------------------------------------------
// GPL_Data_Point::operator=
//
//   Description:
//      Assignment operator
//
//   Params:
//      const GPL_Data_Point &right
//
//   Returns:
//      const GPL_Data_Point &
//--------------------------------------------------------------------
const GPL_Data_Point & GPL_Data_Point::operator=(const GPL_Data_Point &right)
{
    status      = right.status;
    dummy       = right.dummy;
    lattitude   = right.lattitude;
    longitude   = right.longitude;
    altitude    = right.altitude;
    heading     = right.heading;
    speed       = right.speed;
    time        = right.time;
    dummy3      = right.dummy3;

    return *this;
} // operator=

//--------------------------------------------------------------------
// GPL_Data_Point::operator+
//
//   Description:
//      Assignment operator
//
//   Params:
//      const GPL_Data_Point &right
//
//   Returns:
//      const GPL_Data_Point &
//--------------------------------------------------------------------
const GPL_Data_Point & GPL_Data_Point::operator+(const GPL_Data_Point &right)
{
//  status      += right.status;
//  dummy       += right.dummy;
    lattitude   += right.lattitude;
    longitude   += right.longitude;
    altitude    += right.altitude;
    heading     += right.heading;
    speed       += right.speed;
    time        += right.time;
//  dummy3      += right.dummy3;

    return *this;
} // operator+=

//--------------------------------------------------------------------
// GPL_Data_Point::operator-
//
//   Description:
//      Assignment operator
//
//   Params:
//      const GPL_Data_Point &right
//
//   Returns:
//      const GPL_Data_Point &
//--------------------------------------------------------------------
const GPL_Data_Point & GPL_Data_Point::operator-(const GPL_Data_Point &right)
{
//  status      -= right.status;
//  dummy       -= right.dummy;
    lattitude   -= right.lattitude;
    longitude   -= right.longitude;
    altitude    -= right.altitude;
    heading     -= right.heading;
    speed       -= right.speed;
    time        -= right.time;
//  dummy3      -= right.dummy3;

    return *this;
} // operator-=

//--------------------------------------------------------------------
// GPL_Data_Point::operator/
//
//   Description:
//      Assignment operator
//
//   Params:
//      const GPL_Data_Point &right
//
//   Returns:
//      const GPL_Data_Point &
//--------------------------------------------------------------------
const GPL_Data_Point & GPL_Data_Point::operator/(const double val)
{
    lattitude   /= val;
    longitude   /= val;
    altitude    /= val;
    heading     /= val;
    speed       /= val;
    time        = (ULong) ((double) time / val);

    return *this;
} // operator/=

//--------------------------------------------------------------------
// GPL_Data_Point::operator*
//
//   Description:
//      Assignment operator
//
//   Params:
//      const GPL_Data_Point &right
//
//   Returns:
//      const GPL_Data_Point &
//--------------------------------------------------------------------
const GPL_Data_Point & GPL_Data_Point::operator*(const double val)
{

    lattitude   *= val;
    longitude   *= val;
    altitude    *= val;
    heading     *= val;
    speed       *= val;
    time        = (ULong) ((double) time * val);

    return *this;
} // operator*=

//--------------------------------------------------------------------
// GPL_Data_Point::ExtractFromString(...)
//
//   Description:
//      Extract data from a string.  It is assumed the string was
//      created using our operator<< function, or atleast in the same
//      format.
//
//   Params:
//      Std::String     &str    - the string to extract the data from
//
//   Returns:
//      bool    true    - extraction worked (atleast lat & long found)
//              false   - failed
//--------------------------------------------------------------------
bool GPL_Data_Point::ExtractFromString(std::string &str)
{
   bool result = false;
   ULong        pointNum;
   char  timeStr[16];
   char dateStr[12];

   // Set defaults
   status       = 0;
   dummy        = 0;
   lattitude    = 500.0;
   longitude    = 500.0;
   altitude     = 0.0;
   heading      = 0.0;
   speed        = 0.0;
   time = 0;
   dummy3       = 0;

   int howMuch = sscanf(str.c_str(), "%lu:%lu,%lf,%lf,%12s%9s%lf,%lf,%lf",
                        &pointNum, &status, &lattitude, &longitude, dateStr,
                        timeStr, &heading, &speed, &altitude);

   if (howMuch == EOF)                          // did we finish?
   {                                            // ... no, see if we got enough
      if ((lattitude == 500.0) || (longitude == 500.0)) // valid?
         result = false;
      else
         result = true;
   }
   else if (howMuch < 4)                        // scan too few?
      result = false;
   else                                 // scanned just enough
   {                                            // ... so process it
      // convert dateStr and timeStr
      struct  tm data;

      sscanf(timeStr, "%d:%d:%d", &data.tm_hour, &data.tm_min, &data.tm_sec);
      // get the month string
      sscanf(dateStr, "%d", &data.tm_mday);
      sscanf(&dateStr[7], "%d", &data.tm_year);
      data.tm_year -= 1900;

      // convert month text to date
      dateStr[6]='\0';                    // terminate month
      const char *monStr = &dateStr[3];

      if (stricmp(monStr,"JAN") == 0)
         data.tm_mon = 0;
      else if (stricmp(monStr,"FEB") == 0)
         data.tm_mon = 1;
      else if (stricmp(monStr,"MAR") == 0)
         data.tm_mon = 2;
      else if (stricmp(monStr,"APR") == 0)
         data.tm_mon = 3;
      else if (stricmp(monStr,"MAY") == 0)
         data.tm_mon = 4;
      else if (stricmp(monStr,"JUN") == 0)
         data.tm_mon = 5;
      else if (stricmp(monStr,"JUL") == 0)
         data.tm_mon = 6;
      else if (stricmp(monStr,"AUG") == 0)
         data.tm_mon = 7;
      else if (stricmp(monStr,"SEP") == 0)
         data.tm_mon = 8;
      else if (stricmp(monStr,"OCT") == 0)
         data.tm_mon = 9;
      else if (stricmp(monStr,"NOV") == 0)
         data.tm_mon = 10;
      else if (stricmp(monStr,"DEC") == 0)
         data.tm_mon = 11;
      else
         data.tm_mon = 0xFF;

      time = mktime(&data);                     // convert to time_t

      result = true;
   } // else ...

   return result;
} // ExtractFromString(...)
