// Time-stamp: <02-Sep-2002 18:49:32 jwerner>
/*====================================================================
//
// gpldata.h - GPL Data Point Class
//
//     by: John Werner x25054
//	   22-Oct-1997
//
//====================================================================
//
// Purpose:
//
// System Subsystem:
//	   Engineer: John E. Werner
//
//--------------------------------------------------------------------
//
//  Copyright (C) 1997, 1998, 2001, 2002 John E. Werner, All Rights Reserved.
//
// Copyright protection claimed includes all forms and matters of
// copyrightable material and information now allowed by statutory or
// judicial law or hereinafter granted, including without limitation,
// material generated from the software program which are displayed on
// the screen such as icons, screen display looks, etc.
//
//--------------------------------------------------------------------
//
// Revision History:
//   22-Oct-1997  JEWerner	Created.
//
//------------------------------------------------------------------*/
#ifndef GPLDATA_H_INCLUDED
#define GPLDATA_H_INCLUDED

#pragma pack(1)

#include <iostream.h>
#include <string>
typedef unsigned short UShort;
typedef unsigned long  ULong;

//--------------------------------------------------------------------
// Class GPL_Data_Point
//
//   Description:
//	A data point from the GPL file.
//
//   Inherits from:	none
//
//   Revision History:
//	19-Aug-1997	JWerner Created.
//--------------------------------------------------------------------
class GPL_Data_Point
{
  private:
    ULong	status;
    ULong	dummy;
    double	lattitude;
    double	longitude;
    double	altitude;
    double	heading;
    double	speed;
    ULong	time;				// in <time.h> format
    ULong	dummy3;

  public:
    // Constructors
    GPL_Data_Point();
    GPL_Data_Point(double lat, double lon);
    GPL_Data_Point(double lat, double lon, double hght);
    GPL_Data_Point(double lat, double lon, double hght, double head);
    GPL_Data_Point(double lat, double lon, double hght, double head, double
		   spd);
    GPL_Data_Point(double lat, double lon, double hght, double head, double
		   spd, ULong t);
    GPL_Data_Point(double lat, double lon, double hght, double head, double
		   spd, ULong t, ULong status);
    GPL_Data_Point(const GPL_Data_Point &right);

    // Comparison
    int operator== (const GPL_Data_Point& right) const;
    int operator!= (const GPL_Data_Point &right) const;

    // Arithmetic
    const GPL_Data_Point & operator=(const GPL_Data_Point &right);
    const GPL_Data_Point & operator+(const GPL_Data_Point &right);
    const GPL_Data_Point & operator-(const GPL_Data_Point &right);
    const GPL_Data_Point & operator/(const double val);
    const GPL_Data_Point & operator*(const double val);

    // Data Access
    ULong  GetStatus()   	{ return  status   ; };
    double GetLattitude()	{ return  lattitude; };
    double GetLongitude()	{ return  longitude; };
    double GetAltitude() 	{ return  altitude ; };
    double GetHeading()  	{ return  heading  ; };
    double GetSpeed()    	{ return  speed    ; };
    ULong  GetTime()     	{ return  time     ; };

    void SetStatus   (ULong  v)	{ status    = v; };
    void SetLattitude(double v)	{ lattitude = v; };
    void SetLongitude(double v)	{ longitude = v; };
    void SetAltitude (double v)	{ altitude  = v; };
    void SetHeading  (double v)	{ heading   = v; };
    void SetSpeed    (double v)	{ speed     = v; };
    void SetTime     (ULong  v)	{ time      = v; }; // in <time.h> format

    // general functions
    bool SamePlace (const GPL_Data_Point&) const;
    friend ostream& operator<< (ostream& os, const GPL_Data_Point&);
    bool ExtractFromString(std::string & str);
};
#endif /* GPLDATA_H_INCLUDED */
