// Time-stamp: <20-Aug-2001 11:56:45 jwerner>
/*====================================================================
//
// graphgpl.cc - Graph the output of a GPL file
//
//     by: John Werner x25054
//         22-Oct-1997
//
//====================================================================
//
// Purpose:
//
// System Subsystem: 
//         Engineer: John E. Werner
//
//--------------------------------------------------------------------
//
//  Copyright (C) 1997, 1998, 2001 John E. Werner, All Rights Reserved.
//
// Copyright protection claimed includes all forms and matters of
// copyrightable material and information now allowed by statutory or
// judicial law or hereinafter granted, including without limitation,
// material generated from the software program which are displayed on
// the screen such as icons, screen display looks, etc.
//
//--------------------------------------------------------------------
//
// Revision History:
//   22-Oct-1997  JEWerner	Created.
//
//------------------------------------------------------------------*/



/*
* This program can be compiled as follows:
* cc -o example example.c -lGL -lGLU -laux -lXm -lXt -lX11 -lm
*/
#include <GL/glut.h>
#include <iostream.h>
#include <stdlib.h>
#include <io.h>
#include <fcntl.h>
#include <string>
#include <time.h>
#include "gpldata.h"				// the GPL Data Class

GLenum clearMask = GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT;


//--------------------------------------------------------------------
// DrawRoute
//
//   Description:
//	This will actually draw the route for us
//
//   Params:
//	GPL_Data_Point	**	The data points	
//	int	length
//
//   Returns:
//	int	0	- no Error
//--------------------------------------------------------------------
int DrawRoute(GPL_Data_Point *data, int length)
{
    int result = 0;
    GPL_Data_Point	oldPoint;
    GPL_Data_Point	*current;
    
    glClear(clearMask);
    
    glColor3f(0.7 0.5 0.8);    
    glBegin(LINE_STRIP);
    
    for (current = *data; current < data + length; current++)
    {
	if (!oldPoint.SamePlace(*current))
	{
	    glVertex3d(current->longitude, current->lattitude,
		       current->altitude, current->speed);
	}
	oldPoint = *current;
    } // for (...)
    
    glEnd();
    
  
    return result;
} // DrawRoute(...)

//--------------------------------------------------------------------
// main(...)
//
//   Description:
//	Our program's main function
//
//   Params:
//      int     argc    - how many arguments
//      char    **argv  - the arguments
//
//   Returns:
//      int     0       - no Error
//              -1      - file open error
//              -2      - usage error
//--------------------------------------------------------------------
int main(int argc, char **argv)
{
    if (argc < 2)
    {
	cout << "Ussage: graphgpl <file1> <file2> .... <filen>" << endl;
	return(-2);
    }


    readGPL();
    

    // Init OpenGL stuffs
    glClearColor(0.0, 0.0, 0.0, 0.0);
    glShadeModel(GL_FLAT);
    glFrontFace(GL_CW);
    glEnable(GL_DEPTH_TEST);
    glMatrixMode(GL_MODELVIEW);
    glTranslatef(0.0, 0.0, 0.0);
    glScalef(1.0, 1.0, 1.0);
    glRotatef(0.0, 0.0, 0.0, 0.0);


} // main(...)
