// Time-stamp: <05-Sep-2002 13:22:25 jwerner>
/*====================================================================
//
// readgpl.cc - Read DeLorme GPL Files
//
//     by:
//         19-Aug-1997
//
//====================================================================
//
// Purpose:
//
// System Subsystem:
//         Engineer: John E. Werner
//
//--------------------------------------------------------------------
//
//  Copyright (C) 1997, 1998, 2000, 2001, 2002 John Werner, All Rights Reserved.
//
// Copyright protection claimed includes all forms and matters of
// copyrightable material and information now allowed by statutory or
// judicial law or hereinafter granted, including without limitation,
// material generated from the software program which are displayed on
// the screen such as icons, screen display looks, etc.
//
//--------------------------------------------------------------------
//
// Revision History:
//   19-Aug-1997  JEWerner      Created.
//
//------------------------------------------------------------------*/

#pragma pack(1)

#include <iostream.h>
#include <stdlib.h>
#include <stdio.h>
//#include <io.h>
#include <fcntl.h>
#include <string>
#include <time.h>
#include "gpldata.h"
typedef unsigned short UShort;
typedef unsigned long  ULong;

#ifndef bool
//enum bool       {false = 0, true = 1};
#endif

#define READ_SIZE       100


//--------------------------------------------------------------------
// main(...)
//
//   Description:
//      This is the maint function in the program.
//
//   Params:
//      int     argc    - how many arguments
//      char    **argv  - the arguments
//
//   Returns:
//      int     0       - no Error
//              -1      - file open error
//              -2      - usage error
//--------------------------------------------------------------------

int main(int argc, char **argv)
{
   if (argc < 2)
   {
      cout << "Usage: readgpl [-i] <file1> <file2> .... <filen>" << endl
           << endl
           << "\t -i	ignore same place, (i.e. pass same place data points)"
           << endl;
      return(-2);
   }

   std::string      fileName;
   int 	j;
   bool	ignoreSame = false;
   std::string	ignoreFlag("-i");
   int 	returnResult = 0;

   for (j = 1; j < argc; j++)
   {
      fileName = argv[j];

      if (fileName == ignoreFlag)		// see if ignore flag set
      {
         ignoreSame = true;			// mark to ignore
         continue;				// continue with next file
      }

      cout << "# Opening " << fileName << " (" << j << ")" << endl;

      FILE *handle = fopen (fileName.c_str(), "rw");
      if (handle == NULL)
      {
         cout << "\"" << fileName << "\" - Open Failed!!!" << endl;
         returnResult = -1;
         continue;
      }

      bool    done = false;
      int     num = 1;

      cout << "Number:\tstat, lattitude, longitude, date, time, heading, "
         "speed, altitude" << endl;

      while (!done)
      {
         GPL_Data_Point      lastPoint;
         GPL_Data_Point      data[READ_SIZE];
         int size = fread(data, sizeof(GPL_Data_Point), READ_SIZE, handle);
         int number = size;

         for (GPL_Data_Point *dataPntr = data;
              dataPntr < data + number;
              dataPntr++, num++)
         {
            // check to see if we have new place and output
            if (ignoreSame | (!dataPntr->SamePlace(lastPoint)))
               cout << num << ":\t" << *dataPntr << endl;

            lastPoint = *dataPntr;

            if (feof(handle))
               done = true;
         }
      } // while (!done)
      cout << "# ... done.  Processed " << num << " data points."  << endl;
      fclose (handle);
   } // for (j = 1; ...)

   return(returnResult);
} // main(...)
