// Time-stamp: <05-Sep-2002 13:22:27 jwerner>
/*====================================================================
//
// retime.cc - Re-time a GPL file to a standard time
//
//     by: John E. Werner
//         03-Feb-1998
//
//====================================================================
//
// Purpose:
//
// System Subsystem: 
//         Engineer: John E. Werner
//
//--------------------------------------------------------------------
//
//  Copyright (C) 1998, 2001, 2002 John Werner, All Rights Reserved.
//
// Copyright protection claimed includes all forms and matters of
// copyrightable material and information now allowed by statutory or
// judicial law or hereinafter granted, including without limitation,
// material generated from the software program which are displayed on
// the screen such as icons, screen display looks, etc.
//
//--------------------------------------------------------------------
//
// Revision History:
//   03-Feb-1998  JEWerner	Created.
//
//------------------------------------------------------------------*/
#pragma pack(1)

#include <iostream.h>
#include <fstream.h>
#include <string>
#include <stdlib.h>
#include "gpldata.h"
#include <time.h>

typedef unsigned short UShort;
typedef unsigned long  ULong;


//--------------------------------------------------------------------
// main(...)
//
//   Description:
//	The main execution loop
//
//   Params:
//	int	argc	- the number of arguments
//	char	**argv	- the argnuments
//
//   Returns:
//	int	0	No Error
//		!0	Error
//			-1 	Failed to open input file
//			-2	Failed to open output file
//			-3 	Invalid arguments
//--------------------------------------------------------------------

int main(int argc, char **argv)
{
   if ((argc < 3) || (argc > 4))		// check number of args
   {
      cout << "Usage: retime <input.gpl> <output.gpl> [<seconds>]"
           << endl
           << endl
           << "where:\t<input.gpl>   is the GPL file to read" << endl
           << "\t<output.gpl>  is the file to write" << endl
           << "\t<stop speed>  is the number of seconds to increment by (1)"
           << endl
           << endl
           << "Example:\tretime input.gpl output.gpl 3" << endl
           << "\t This reads from input.gpl, writes output.gpl, with 3" 
           << endl << "seconds between each data point." << endl;
      return -3;
   }
   time_t	interval = 1;    

   if (argc == 4)				// interval given?
      interval = strtoul(argv[3], NULL, 0);	// get time step

   std::string      outName(argv[2]), inName(argv[1]);

   cout << "Opening " << inName << endl;
   ifstream in(inName.c_str(), ios::binary);
   if (!in)
   {
      cout << "\"" << inName << "\" - Open Failed!!!" << endl;
      return -1;
   }

   cout << "Opening " << outName << endl;
   ofstream out(outName.c_str(), ios::binary | ios::trunc);
   if (!out)
   {
      cout << "\"" << outName << "\" - Open Failed!!!" << endl;
      in.close();
      return(-2);
   }

   cout << "Working...";
   bool done = false;
   int  num = 1;
   time_t	ourTime;
   ourTime = time(NULL);		// get the time now

   GPL_Data_Point data;
   while (!done)
   {
//      bool writeIt = false;

      if (in.eof())				// see if input file done
      {
         done = true;
         break;
      }

      // get data point
      in.read((char *) &data, sizeof(data));
      if (in.fail())
      {
         done = true;
         break;
      }

      data.SetTime(ourTime);
      ourTime += interval;
	
      out.write((char *) &data, sizeof(data)); // ... yes, so write it

      num++;
      if (num % 100 == 0)
         cout << ".";

   } // while(!done)

   // close stuff and leave
   in.close();
   out.close();

   cout << "done." << endl;
   cout << "Wrote " << num << " data points." << endl;

   return 0;
} // main(...)
