/************************************************************************

Project:      Cetus GPS track export
Description:  Header file containing defines for the TrackLog program.
File:         track.h
Author:       Kjeld Jensen <kjeld@cetus.dk>
Homepage:     http://www.cetus.dk/Cetus GPS

Revision history:

Date     Vers  Author Description
-------- ----- ------ ---------------------------------------------------
01-06-01  1.0    KJ   Created, supports Cetus GPS 1.0
01-06-01  1.01   KJ   Added track_export return codes
26-06-01  1.10   KJ   Changed the track record structure and added
                      optional formatting of position, altitude, speed,
                      data not available and description.
26-07-01  1.20   KJ   Added utc variable to the init_type.

*************************************************************************
COPYRIGHT

Cetus GPS is copyrighted (c) 2001 by

Kjeld Jensen
Niels Bohrs Alle 23, apt. 2006
DK-5230 Odense M
Denmark
Email: gps@cetus.dk
http://www.cetus.dk/gps


COPYRIGHT INFORMATION
This source code is a part of the Cetus GPS project. The copyright stated
above includes all parts of the Cetus GPS project including but not
limited to binaries, source code, databases, documentation, homepage
information and design.

The following copyright information applies to the source code routines
published for the purpose of Cetus GPS data import and export and
binaries that includes a part of this source code. It does not apply to
the Cetus GPS project in general or other parts of the Cetus GPS project.

Redistribution of this source code for commercial or non-commercial use
in source and binary forms, with or without modification, are permitted
provided that the following conditions are met:

1. Copyright of the source code remain Kjeld Jensen's and as such any
   copyright notices in the source code are not to be removed.

2. Redistributions of the source code with or without modifications must
   retain the copyright notice, this source code copyright information
   and the disclaimer.

3. The licence and distribution terms for any available version or
   derivative of this source code cannot be changed. i.e. this source
   code cannot be copied and put under any other distribution licence.

4. The product documentation must include an acknowledgement where
   appropriate. A paragraph similar to the following is acceptable:
   "This product includes software written by Kjeld Jensen
   <kjeld@cetus.dk>"

Revised July 1, 2001


DISCLAIMER
All parts of the Cetus GPS project are provided by the copyright holder
as is, and any express or implied warranties, including, but not limited
to, the implied warranties of merchantability and fitness for a
particular purpose are disclaimed.

In no event skal the author or contributors be liable for any direct,
indirect, incidental, special, exemplary or consequential damages
(including but not limited to procurement of substitute goods or
services, loss of use, data, or profits, or business interruption)
however caused and on any theory of liability, whether in contract,
strict liability, or tort (including negligence or otherwise) arising
in any way out of the use of any parts of the Cetus GPS project, even
if advised of the possibility of such damage.

Revised July 1, 2001

************************************************************************/


/***********************************************************************/
/* Data structures */
/* see tracklog.h for a detailed description of the variables */
#include <fstream>

struct init_type
{
	int				interval;
	short			gps;
	short			mag;
	unsigned int	year;
	unsigned int	month;
	unsigned int	day;
	unsigned int	hour;
	unsigned int	minute;
	unsigned int	second;
    unsigned int    decSecond;
	int				timeZone;
	unsigned int    datum;
	unsigned int    north;
	char			desc[30];
};


struct rec_type
{
	unsigned int	fix;
	unsigned int	year;
	unsigned int	month;
	unsigned int	day;
	unsigned int	hour;
	unsigned int	minute;
	unsigned int	second;
        unsigned int    decSecond;
	unsigned int	sat;
	unsigned int	hdop;
	long		lat;
	long		lon;
	long		altitude;
	unsigned short	speed;
	unsigned short	course;    
};

/***********************************************************************/
/* Prototypes */

int track_init (ifstream &in, struct init_type *init);


/************************************************************************

track_init return values:

	0   Ok
	1   track file not found
	2   file is not a track file

************************************************************************/

int track_record (ifstream &in, struct rec_type &rec);
