// Time-stamp: <05-Sep-2002 13:22:27 jwerner>
/*====================================================================
//
// txt2gpl.cc - Convert a text file (like we generate) to GPL file
//
//     by: John E. Werner
//         02-Feb-1998
//
//====================================================================
//
// Purpose:
//
// System Subsystem:
//         Engineer: John E. Werner
//
//--------------------------------------------------------------------
//
//  Copyright (C) 1998, 2000, 2001, 2002 John Werner, All Rights Reserved.
//
// Copyright protection claimed includes all forms and matters of
// copyrightable material and information now allowed by statutory or
// judicial law or hereinafter granted, including without limitation,
// material generated from the software program which are displayed on
// the screen such as icons, screen display looks, etc.
//
//--------------------------------------------------------------------
//
// Revision History:
//   02-Feb-1998  JEWerner	Created.
//
//------------------------------------------------------------------*/
#pragma pack(1)

#include <iostream.h>
#include <fstream.h>
#include <string>
#include "gpldata.h"

typedef unsigned short UShort;
typedef unsigned long  ULong;

//--------------------------------------------------------------------
// main(...)
//
//   Description:
//	The main program file
//
//   Params:
//	void
//
//   Returns:
//	int	0	no error
//		!0	Error
//			-1 	Failed to open input file
//			-2	Failed to open output file
//			-3	Invalid arguments
//--------------------------------------------------------------------

int main(int argc, char **argv)
{
   if (argc != 3)
   {
      cout << "Usage: txt2gpl <input file.TXT> <output file.GPL>" << endl;
      return(-3);
   }

   std::string      outName, inName;

   inName = argv[1];

   cout << "Opening " << inName << endl;
   ifstream in(inName.c_str());
   if (!in)
   {
      cout << "\"" << inName << "\" - Open Failed!!!" << endl;
      return -1;
   }


   outName = argv[2];				// get the file name
   cout << "Opening " << outName << endl;
   ofstream out(outName.c_str(), ios::binary | ios::trunc);
   if (!out)
   {
      cout << "\"" << outName << "\" - Open Failed!!!" << endl;
      in.close();
      return(-2);
   }

   cout << "Working...";
   bool done = false;
   int  num = 1;
   while (!done)
   {
      if (in.eof())				// see if input file done
      {
         done = true;
         break;
      }

      char inStr[512];
      in.getline(inStr, 511, '\n');		// read a line of the input

      GPL_Data_Point	data;
      std::string tempStr(inStr);	
      if (data.ExtractFromString(tempStr))	// see if valid data
      {					// ... yes, so write it out
         out.write((char *) &data, sizeof(data));
         out.flush();
      }
	
      num++;
      if (num % 100 == 0)
         cout << ".";

   } // while(!done)

   // close stuff and leave
   in.close();
   out.close();

   cout << "done." << endl;
   cout << "Wrote " << num << " data points." << endl;

   return 0;
} // main(...)
